import numpy as np
import matplotlib.pyplot as plt
import skimage.io as io
import os
from first_diff import opDx
from first_diff import opDtx

def gamma_correct(img):
    return np.where(img <= 0.0031308, \
                    12.92 * img,     \
                    (1.055) * np.power(img,(1.0/2.4)) - 0.055)

def compute_grad_mag(img):
    Dx = opDx(img, axis=-1)
    Dy = opDx(img, axis=-2)
    return np.sum(np.sqrt(Dx*Dx + Dy*Dy), axis=0)

def plot_gm(img, fname):
    min = np.min(img)
    max = np.max(img)
    print(fname + ": ", min, max)
    count = np.where(img > 150.0, 1.0, 0.0)
    sum = np.sum(count)
    print(sum, img.size, sum/img.size)
    plotimg = np.where(img > 150.0, 150.0, img) / 150.0
    plotimg = gamma_correct(plotimg)
    io.imsave(fname, (plotimg * 255.0).astype(np.uint8))

def plot_grad_mag(img, name):
    fname = name + '_gradient_magnitude.png'
    plot_gm(img,fname)

def plot_grad_mag_seq(img, grad_mag_dirname, reg_term, lam, seqnum):
    fname = grad_mag_dirname + "/" + str(seqnum).zfill(3) +".png"
    plot_gm(img, fname)

def plot_grad_hist_to_screen(img):
    Dx = opDx(img, axis=-1)
    Dy = opDx(img, axis=-2)
    Dxy = np.concatenate((Dx, Dy), axis=None)
    plt.hist(Dxy, bins=100)
    plt.show()
    plt.close()
    std_dev = np.std(Dxy)
    print("std_dev: ", std_dev)
    # h = np.histogram(Dxy,bins=10)
    # print(h)

def plot_grad_hist_to_file(img, grad_hist_dirname, title, filenm, seqnum):
    Dx = opDx(img, axis=-1)
    Dy = opDx(img, axis=-2)
    Dxy = np.concatenate((Dx, Dy), axis=None)
    plt.figure()
    plt.hist(Dxy, bins=100)
    plt.title(title + ", Iteration " + str(seqnum))
    fname = grad_hist_dirname + "/" + filenm + "_" + \
        str(seqnum).zfill(3) +".png"
    plt.savefig(fname, bbox_inches='tight')
    plt.close()


def plot_hist_to_screen(img):
    tmp = np.reshape(img, img.size)
    plt.hist(tmp, bins=200)
    plt.show()
    plt.close()
    std_dev = np.std(tmp)
    print("std_dev: ", std_dev)

def plot_hist_to_file(img, dirname, title, filenm, seqnum):
    tmp = np.reshape(img, img.size)
    plt.figure()
    plt.hist(tmp, bins=200)
    plt.title(title + ", Iteration " + str(seqnum))
    fname = dirname + "/" + filenm + "_" + \
        str(seqnum).zfill(3) +".png"
    plt.savefig(fname, bbox_inches='tight')
    plt.close()

def compute_gradient_std_dev(Dx, Dy):
    Dxy = np.concatenate((Dx, Dy), axis=None)
    return np.std(Dxy)

def compute_pct_under_threshold(img, threshold):
    tmp = np.where(img <= threshold, 1.0, 0.0)
    return 100.0 * (np.sum(tmp) / img.size)

def check_create_dir(dirname):
    checkdir = os.path.isdir(dirname)
    if not checkdir:
        try:  
            os.mkdir(dirname)  
        except OSError as error:  
            print(error)

def compute_ground_truth_stats(x_gt, grad_mag_threshold):
    Dx = opDx(x_gt, axis=-1)
    Dy = opDx(x_gt, axis=-2)
    abs_Dx = np.fabs(Dx)
    abs_Dy = np.fabs(Dy)
    DxyL1norm = np.sum(abs_Dx) + np.sum(abs_Dy)
    gradient_std_dev = compute_gradient_std_dev(Dx, Dy)
    x_grad_mag = compute_grad_mag(x_gt)
    pct_grad_mag_threshold = \
        compute_pct_under_threshold(x_grad_mag, grad_mag_threshold)
    return [DxyL1norm, gradient_std_dev, pct_grad_mag_threshold]


