import numpy as np
from numpy.fft import fft2, ifft2

# I forgot to make python versions of the opDx and opDtx functions available, so here
# they are (I put this into a file called first_diff.py)

# Actually, the functions opDx.m and opDtx.m correspond to grad_fn and t_grad_fn below.
# Sorry for the confusion!

# Note: The equivalent fourier-domain transfer functions are:
# from pypher.pypher import psf2otf
# dx = np.array([[1., -1.]])
# dy = np.array([[1.], [-1.]])
# dxFT = psf2otf(dx, b.shape[-2:])
# dyFT = psf2otf(dy, b.shape[-2:])
# dxTFT = np.conj(dxFT)
# dyTFT = np.conj(dyFT)
# dxyFT = np.stack((dxFT, dyFT), axis=0)
# dxyTFT = np.stack((dxTFT, dyTFT), axis=0)

def opDx(x, axis):
    slc = [slice(None)] * len(x.shape)
    slc[axis] = slice(0, 1)
    Dx = np.diff(x, axis=axis, append=x[tuple(slc)])
    return Dx

def opDtx(diffx, axis):
    Dtx = -diffx + np.roll(diffx, 1, axis=axis)
    return Dtx

def grad_fn(x):
    Dx = opDx(x, axis=-1)
    Dy = opDx(x, axis=-2)
    if len(Dx.shape) <= 3:
        return np.stack((Dx, Dy), axis=-3)
    else:
        return np.concatenate((Dx, Dy), axis=-3)

def t_grad_fn(x):
    Dtx = opDtx(x[..., 0, :, :], axis=-1)
    Dty = opDtx(x[..., 1, :, :], axis=-2)
    return Dtx + Dty

